/* $Id: Pltk_Init.c,v 1.24 1995/10/22 17:31:18 mjl Exp $
 * $Log: Pltk_Init.c,v $
 *
 * Revision 1.25  1996/11/7 vince
 * This file is derived from the old tkshell.c, and implements the Pltk init function.
 * It can be loaded into any Tcl7.5 interpreter, and requires some .tcl library files
 * to be in known places.  Set your environment variables to make sure of this.
 * 
 * Old changes see 'tkshell.c'.  This file should be used _instead_ of tkshell.c,
 * if building the 'libPltk.so' shared library for dynamic loading into Tcl. 
 */

/* 
 * tkshell.c
 * Maurice LeBrun
 * 6-May-93
 *
 * A miscellaneous assortment of Tcl support functions.
 */

#include "plserver.h"

/*----------------------------------------------------------------------*\
 * Pltk_Init
 *
 * Initialization routine for extended wish'es.
 * Creates the plframe, matrix, wait_until, and host_id (w/Tcl-DP only)
 * commands.  Also sets the auto_path variable.
\*----------------------------------------------------------------------*/

/*
 * Default directory in which to look for libraries:
 */

#ifndef PL_LIBRARY
#define PL_LIBRARY ""
#endif

static char defaultLibraryDir[200] = PL_LIBRARY;

int
Plplot_Init( Tcl_Interp *interp )
{
    static char initScript[] =
    "set pldirs {}\n\
    if [info exists env(PL_LIBRARY)] {\n\
	lappend pldirs $env(PL_LIBRARY)\n\
    }\n\
    if [info exists env(EXT_FOLDER)] {\n\
	lappend pldirs [file join $env(EXT_FOLDER) \"Tool Command Language\" lib plplot$plversion]\n\
    }\n\
    lappend pldirs $pllibrary\n\
    set pllibrary {}\n\
    if ![catch {uplevel #0 source -rsrc plplot}] {\n\
	return\n\
    }\n\
    lappend pldirs [file join [file dirname [info library]] plplot$plversion]\n\
    set parentDir [file dirname [file dirname [info nameofexecutable]]]\n\
    lappend pldirs [file join $parentDir lib plplot$plversion]\n\
    lappend pldirs [file join [file dirname $parentDir] plplot library]\n\
    lappend pldirs [file join $parentDir library]\n\
    foreach i $pldirs {\n\
	set pllibrary $i\n\
	if ![catch {uplevel #0 source \\{[file join $i plplot.tcl]\\}}] {\n\
	    break\n\
	}\n\
	set pllibrary {}\n\
    }\n\
    if {$pllibrary == {}} {\n\
    set msg \"Can't find a usable plplot.tcl in the following directories: \n\"\n\
    append msg \"    $pldirs\n\"\n\
    append msg \"This probably means that Plplot wasn't installed properly\n\"\n\
    append msg \"or you need to set your PL_LIBRARY environment variable?\"\n\
    error $msg\n\
    }\n\
    set plversion\n\
    ";

    char *libDir;
    Tcl_DString path;

    Tk_Window main;

    if (Tcl_PkgRequire(interp, "Tk", TK_VERSION, 0) == NULL) {
	return TCL_ERROR;
    }
	
    main = Tk_MainWindow(interp);

/* plframe -- PLplot graphing widget */

    Tcl_CreateCommand(interp, "plframe", plFrameCmd,
                      (ClientData) main, (void (*)(ClientData)) NULL);

/* wait_until -- waits for a specific condition to arise */
/* Can be used with either Tcl-DP or TK */

    Tcl_CreateCommand(interp, "wait_until", plWait_Until,
		      (ClientData) NULL, (void (*) (ClientData)) NULL);

/* host_id -- returns host IP number.  Only for use with Tcl-DP */

#ifdef PLD_dp
    Tcl_CreateCommand(interp, "host_id", plHost_ID,
		      (ClientData) NULL, (void (*) (ClientData)) NULL);
#endif

/* Set up auto_path */

    if (pls_auto_path(interp) == TCL_ERROR)
	return TCL_ERROR;

/* Save initial RGB colormap components */
/* Disabled for now */

#if 0
    {
    Display *display;
    Colormap map;

    display = Tk_Display(main);
    map = DefaultColormap(display, DefaultScreen(display));

    PLX_save_colormap(display, map);
    }
#endif

    if(Tcl_PkgRequire(interp,"Matrix","0.1",0) == NULL)
		return TCL_ERROR;

    Tcl_SetVar(interp, "plversion", "4.99", TCL_GLOBAL_ONLY);
    libDir = Tcl_GetVar(interp, "pllibrary", TCL_GLOBAL_ONLY);
    if (libDir == NULL) {
	Tcl_SetVar(interp, "pllibrary", defaultLibraryDir, TCL_GLOBAL_ONLY);
    }

    Tcl_PkgProvide(interp,"Plplot","4.99");

    return Tcl_Eval(interp, initScript);
}


/*----------------------------------------------------------------------*\
 * plWait_Until
 *
 * Tcl command -- wait until the specified condition is satisfied.
 * Processes all events while waiting.
 *
 * This command is more capable than tkwait, and has the added benefit
 * of working with Tcl-DP as well.  Example usage:
 *
 *	wait_until {[info exists foobar]}
 *
 * Note the [info ...] command must be protected by braces so that it
 * isn't actually evaluated until passed into this routine.
\*----------------------------------------------------------------------*/

int
plWait_Until(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
    int result = 0;

    dbug_enter("plWait_Until");

    for (;;) {
	if (Tcl_ExprBoolean(interp, argv[1], &result)) {
	    fprintf(stderr, "wait_until command \"%s\" failed:\n\t %s\n",
		    argv[1], interp->result);
	    break;
	}
	if (result)
	    break;

	Tcl_DoOneEvent(0);
    }
    return TCL_OK;
}
